@@ -24,16 +24,16 @@
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
  *
- * @property Horde_ActiveSync_Wbxml_Encoder encoder The Wbxml encoder.
- * @property Horde_ActiveSync_Wbxml_Decoder decoder The Wbxml decoder.
- * @property Horde_ActiveSync_State_Base state      The state object.
- * @property Horde_Controller_Reqeust_Http request  The HTTP request object.
- * @property Horde_ActiveSync_Driver_Base driver
- * @property boolean provisioning
- * @property boolean multipart
- * @property string certPath
- * @property Horde_ActiveSync_Device device
- * @property Horde_Log_Logger logger
+ * @property-read Horde_ActiveSync_Wbxml_Encoder $encoder The Wbxml encoder.
+ * @property-read Horde_ActiveSync_Wbxml_Decoder $decoder The Wbxml decoder.
+ * @property-read Horde_ActiveSync_State_Base $state      The state object.
+ * @property-read Horde_Controller_Reqeust_Http $request  The HTTP request object.
+ * @property-read Horde_ActiveSync_Driver_Base $driver    The backend driver object.
+ * @property-read boolean|string $provisioning Provisioning support: True, False, or 'loose'
+ * @property-read boolean $multipart Indicate this is a multipart request.
+ * @property-read string $certPath Local path to the certificate bundle.
+ * @property-read Horde_ActiveSync_Device $device  The current device object.
+ * @property-read Horde_Log_Logger $logger   The logger object.
  */
 class Horde_ActiveSync
 {
@@ -289,7 +289,7 @@ class Horde_ActiveSync
     const AUTH_REASON_USER_DENIED               = 'user';
     const AUTH_REASON_DEVICE_DENIED             = 'device';
 
-    const LIBRARY_VERSION                       = '2.12.3';
+    const LIBRARY_VERSION                       = '2.16.11';
 
     /**
      * Logger
@@ -308,7 +308,7 @@ class Horde_ActiveSync
     /**
      * Provisioning support
      *
-     * @var string (TODO _constant this)
+     * @var string
      */
     protected $_provisioning;
 
@@ -513,31 +513,14 @@ class Horde_ActiveSync
     /**
      * Authenticate to the backend.
      *
+     * @param Horde_ActiveSync_Credentials $credentials  The credentials object.
+     *
      * @return boolean  True on successful authentication to the backend.
      * @throws Horde_ActiveSync_Exception
      */
-    public function authenticate($username = '')
+    public function authenticate(Horde_ActiveSync_Credentials $credentials)
     {
-        // Get credentials
-        $serverVars = $this->_request->getServerVars();
-        $user = $pass = '';
-        if (!empty($serverVars['PHP_AUTH_PW'])) {
-            $user = empty($username) ? $serverVars['PHP_AUTH_USER'] : $username;
-            $pass = $serverVars['PHP_AUTH_PW'];
-        } elseif (!empty($serverVars['HTTP_AUTHORIZATION']) || !empty($serverVars['Authorization'])) {
-            // Some clients use the non-standard 'Authorization' header.
-            $authorization = !empty($serverVars['HTTP_AUTHORIZATION'])
-                ? $serverVars['HTTP_AUTHORIZATION']
-                : $serverVars['Authorization'];
-            $hash = base64_decode(str_replace('Basic ', '', $authorization));
-            if (strpos($hash, ':') !== false) {
-                list($user, $pass) = explode(':', $hash, 2);
-            }
-            $user = !empty($username) ? $username : $user;
-        } elseif (!empty($username)) {
-            // Might not need a password, could be using X509 cert.
-            $user = $username;
-        } else {
+        if (!$credentials->username) {
             // No provided username or Authorization header.
             self::$_logger->notice(sprintf(
                 '[%s] Client did not provide authentication data.',
@@ -545,7 +528,8 @@ class Horde_ActiveSync
             );
             return false;
         }
-        $user = $this->_driver->getUsernameFromEmail($user);
+
+        $user = $this->_driver->getUsernameFromEmail($credentials->username);
         $pos = strrpos($user, '\\');
         if ($pos !== false) {
             $domain = substr($user, 0, $pos);
@@ -555,7 +539,7 @@ class Horde_ActiveSync
         }
 
         // Authenticate
-        if ($result = $this->_driver->authenticate($user, $pass, $domain)) {
+        if ($result = $this->_driver->authenticate($user, $credentials->password, $domain)) {
             if ($result === self::AUTH_REASON_USER_DENIED) {
                 $this->_globalError = Horde_ActiveSync_Status::SYNC_NOT_ALLOWED;
             } elseif ($result === self::AUTH_REASON_DEVICE_DENIED) {
@@ -716,8 +700,7 @@ class Horde_ActiveSync
             $this->_driver->clearAuthentication();
             return $result;
         }
-
-        if (!$this->authenticate(!empty($get['User']) ? $get['User'] : '')) {
+        if (!$this->authenticate(new Horde_ActiveSync_Credentials($this))) {
             $this->activeSyncHeader();
             $this->versionHeader();
             $this->commandsHeader();
@@ -1018,17 +1001,16 @@ class Horde_ActiveSync
     /**
      * Obtain the ActiveSync protocol version requested by the client headers.
      *
-     * @return long
+     * @return string  The EAS version requested by the client.
      */
     public function getProtocolVersion()
     {
-        if (isset(self::$_version)) {
-            return self::$_version;
-        }
-        self::$_version = $this->_request->getHeader('MS-ASProtocolVersion');
-        if (empty(self::$_version)) {
-            $get = $this->getGetVars();
-            self::$_version = empty($get['ProtVer']) ? '1.0' : $get['ProtVer'];
+        if (!isset(self::$_version)) {
+            self::$_version = $this->_request->getHeader('MS-ASProtocolVersion');
+            if (empty(self::$_version)) {
+                $get = $this->getGetVars();
+                self::$_version = empty($get['ProtVer']) ? '1.0' : $get['ProtVer'];
+            }
         }
         return self::$_version;
     }
