@@ -23,6 +23,7 @@
  * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
+ * @internal  Not intended for use outside of the ActiveSync library.
  */
 class Horde_ActiveSync_Collections implements IteratorAggregate
 {
@@ -130,6 +131,13 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      */
     protected $_changes;
 
+    /**
+     * Flag to indicate the client is requesting a hanging SYNC.
+     *
+     * @var boolean
+     */
+    protected $_hangingSync = false;
+
     /**
      * Const'r
      *
@@ -207,6 +215,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
             return $this->_cache->$property;
         case 'importedChanges':
         case 'shortSyncRequest':
+        case 'hangingSync':
             $p = '_' . $property;
             return $this->$p;
         }
@@ -222,6 +231,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         switch ($property) {
         case 'importedChanges':
         case 'shortSyncRequest':
+        case 'hangingSync':
             $p = '_' . $property;
             $this->$p = $value;
             return;
@@ -259,7 +269,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
     }
 
     /**
-     * Add a new populated collection array to this collection.
+     * Add a new populated collection array to the sync cache.
      *
      * @param array $collection        The collection array.
      * @param boolean $requireSyncKey  Attempt to read missing synckey from
@@ -279,6 +289,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                     synckey could be found. Most likely a client error in
                     requesting a collection during PING before it has issued a
                     SYNC.', $this->_procid));
+                throw new Horde_ActiveSync_Exception_StateGone('Synckey required in Horde_ActiveSync_Collections::addCollection, but none was found.');
             }
 
             $this->_logger->info(sprintf(
@@ -291,7 +302,12 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         if (empty($collection['class'])) {
             $collection['class'] = $this->getCollectionClass($collection['id']);
         }
-        $collection['serverid'] = $this->getBackendIdForFolderUid($collection['id']);
+
+        try {
+            $collection['serverid'] = $this->getBackendIdForFolderUid($collection['id']);
+        } catch (Horde_ActiveSync_Exception $e) {
+            throw new Horde_ActiveSync_Exception_StateGone($e->getMessage());
+        }
 
         $this->_collections[$collection['id']] = $collection;
         $this->_logger->info(sprintf(
@@ -319,6 +335,9 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         if ($folder) {
             return $folder['serverid'];
         } else {
+            $this->_logger->err(
+                sprintf('[%s] Horde_ActiveSync_Collections::getBackendIdForFolderUid failed because folder was not found in cache.',
+                $this->_procid));
             throw new Horde_ActiveSync_Exception('Folder not found in cache.');
         }
     }
@@ -538,7 +557,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
      */
     public function canDoLoopingSync()
     {
-        return !$this->_importedChanges && ($this->_shortSyncRequest || $this->_cache->hbinterval !== false || $this->_cache->wait !== false);
+        return $this->_hangingSync && !$this->_importedChanges && ($this->_shortSyncRequest || $this->_cache->hbinterval !== false || $this->_cache->wait !== false);
     }
 
     /**
@@ -595,7 +614,9 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         Horde_ActiveSync_Message_Folder $folder, $update = false)
     {
         $this->_cache->updateFolder($folder);
-
+        $cols = $this->_cache->getCollections(false);
+        $cols[$folder->serverid]['serverid'] = $folder->_serverid;
+        $this->_cache->updateCollection($cols[$folder->serverid]);
         if ($update) {
             $this->_as->state->updateServerIdInState($folder->serverid, $folder->_serverid);
         }
@@ -797,7 +818,7 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
     public function canSendEmptyResponse()
     {
         return !$this->_importedChanges &&
-            ($this->_cache->wait !== false || $this->_cache->hbinterval !== false);
+            ($this->_hangingSync && ($this->_cache->wait !== false || $this->_cache->hbinterval !== false));
     }
 
     /**
@@ -859,9 +880,9 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
             !is_null($filter) &&
             $cc[$id]['filtertype'] != $filter) {
 
-            $this->_cache->removeCollection($id);
+            $this->_cache->removeCollection($id, true);
             $this->_cache->save();
-            $this->_logger->info('Invalidating SYNCKEY - found updated filtertype');
+            $this->_logger->info('Invalidating SYNCKEY and removing collection - found updated filtertype');
 
             return false;
         }
@@ -1076,6 +1097,11 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
                         $this->_procid,
                         $e->getMessage())
                     );
+                    // If we are missing a folder, we should clear the PING
+                    // cache also, to be sure it picks up any hierarchy changes
+                    // since most clients don't seem smart enough to figure this
+                    // out on their own.
+                    $this->resetPingCache();
                     return self::COLLECTION_ERR_FOLDERSYNC_REQUIRED;
 
                 } catch (Horde_ActiveSync_Exception $e) {
@@ -1163,6 +1189,23 @@ class Horde_ActiveSync_Collections implements IteratorAggregate
         }
     }
 
+    /**
+     * Force reset all collection's PINGABLE flag. Used to force client
+     * to issue a non-empty PING request.
+     *
+     */
+    public function resetPingCache()
+    {
+        $collections = $this->_cache->getCollections(false);
+        foreach ($collections as $id => $collection) {
+            $this->_logger->info(sprintf(
+                'UNSETTING collection %s (%s) PINGABLE flag.',
+                $collection['serverid'],
+                $id));
+            $this->_cache->removePingableCollection($id);
+        }
+    }
+
     /**
      * Return the any changes for the current collection, and cache them if
      * we are not PINGing.
