@@ -61,6 +61,7 @@ class Horde_ActiveSync_Imap_Adapter
         Horde_Mime_Part::$defaultCharset = 'UTF-8';
         Horde_Mime_Headers::$defaultCharset = 'UTF-8';
         $this->_procid = getmypid();
+        $this->_logger = new Horde_Support_Stub();
     }
 
     /**
@@ -191,7 +192,7 @@ class Horde_ActiveSync_Imap_Adapter
         $imap = $this->_getImapOb();
         $mbox = new Horde_Imap_Client_Mailbox($mailbox);
         $messages = $this->_getMailMessages($mbox, array($uid));
-        if (!$messages[$uid]->exists(Horde_Imap_Client::FETCH_STRUCTURE)) {
+        if (empty($messages[$uid]) || !$messages[$uid]->exists(Horde_Imap_Client::FETCH_STRUCTURE)) {
             throw new Horde_ActiveSync_Exception('Message Gone');
         }
         $msg = new Horde_ActiveSync_Imap_Message($imap, $mbox, $messages[$uid]);
@@ -536,6 +537,7 @@ class Horde_ActiveSync_Imap_Adapter
      * Attempt to find a Message-ID in a list of mail folders.
      *
      * @return array  An array with the 0 element being the mbox
+     * @throws Horde_Exception_NotFound, Horde_ActiveSync_Exception
      */
     public function getUidFromMidInFolders($id, array $folders)
     {
@@ -543,7 +545,11 @@ class Horde_ActiveSync_Imap_Adapter
         $search_q->headerText('Message-ID', $id);
         foreach ($folders as $folder) {
             $mbox = new Horde_Imap_Client_Mailbox($folder->_serverid);
-            $results = $this->_getImapOb()->search($mbox, $search_q);
+            try {
+                $results = $this->_getImapOb()->search($mbox, $search_q);
+            } catch (Horde_Imap_Client_Exception $e) {
+                throw new Horde_ActiveSync_Exception($e->getMessage());
+            }
             $uid = $results['match']->ids;
             if (!empty($uid)) {
                 return array($mbox, current($uid));
@@ -560,9 +566,7 @@ class Horde_ActiveSync_Imap_Adapter
      * @param array $ids           The message UIDs of the messages to move.
      * @param string $newfolderid  The folder id to move $id to.
      *
-     * @return array  An hash of oldUID => newUID. If the server does not
-     *                support UIDPLUS, then this is a best guess and might fail
-     *                on busy folders.
+     * @return array  An hash of oldUID => newUID.
      *
      * @throws Horde_ActiveSync_Exception
      */
@@ -571,10 +575,6 @@ class Horde_ActiveSync_Imap_Adapter
         $imap = $this->_getImapOb();
         $from = new Horde_Imap_Client_Mailbox($folderid);
         $to = new Horde_Imap_Client_Mailbox($newfolderid);
-        if (!$imap->queryCapability('UIDPLUS')) {
-            $status = $imap->status($to, Horde_Imap_Client::STATUS_UIDNEXT_FORCE);
-            $uidnext = $status[Horde_Imap_Client::STATUS_UIDNEXT];
-        }
         $ids_obj = new Horde_Imap_Client_Ids($ids);
 
         // Need to ensure the source message exists so we may properly notify
@@ -587,26 +587,12 @@ class Horde_ActiveSync_Imap_Adapter
         }
 
         try {
-            $copy_res = $imap->copy($from, $to, array('ids' => $ids_obj, 'move' => true));
+            return $imap->copy($from, $to, array('ids' => $ids_obj, 'move' => true, 'force_map' => true));
         } catch (Horde_Imap_Client_Exception $e) {
-            // We already got rid of the missing ids, this must be something
-            // else.
+            // We already got rid of the missing ids, must be something else.
             $this->_logger->err($e->getMessage());
             throw new Horde_ActiveSync_Exception($e);
         }
-
-        // old_id => new_id
-        if (is_array($copy_res)) {
-            return $copy_res;
-        }
-
-        // No UIDPLUS
-        $ret = array();
-        foreach ($ids_obj->ids as $id) {
-            $ret[$id] = $uidnext++;
-        }
-
-        return $ret;
     }
 
     /**
@@ -900,7 +886,7 @@ class Horde_ActiveSync_Imap_Adapter
 
         // Fill in other header data
         $eas_message->from = $imap_message->getFromAddress();
-        $eas_message->subject = $this->_validateUtf8($imap_message->getSubject(), $hdr_charset);
+        $eas_message->subject = Horde_ActiveSync_Utils::ensureUtf8($imap_message->getSubject(), $hdr_charset);
         $eas_message->threadtopic = $eas_message->subject;
         $eas_message->datereceived = $imap_message->getDate();
         $eas_message->read = $imap_message->getFlag(Horde_Imap_Client::FLAG_SEEN);
@@ -998,7 +984,7 @@ class Horde_ActiveSync_Imap_Adapter
                     $airsync_body->estimateddatasize = $base->getBytes();
                 } else {
                     // Signed/Encrypted message - can't mess with it at all.
-                    $raw = new Horde_ActiveSync_Rfc822($imap_message->getFullMsg(true));
+                    $raw = new Horde_ActiveSync_Rfc822($imap_message->getFullMsg(true), false);
                     $airsync_body->estimateddatasize = $raw->getBytes();
                     $airsync_body->data = $raw->getString();
                     $eas_message->messageclass = 'IPM.Note.SMIME.MultipartSigned';
@@ -1040,21 +1026,25 @@ class Horde_ActiveSync_Imap_Adapter
                 } else {
                     $airsync_body->type = Horde_ActiveSync::BODYPREF_TYPE_HTML;
                 }
-                $airsync_body->estimateddatasize = $message_body_data['html']['estimated_size'];
-                $airsync_body->truncated = $message_body_data['html']['truncated'];
-                $airsync_body->data = $message_body_data['html']['body']->stream;
-                $eas_message->airsyncbasebody = $airsync_body;
+                if (!empty($message_body_data['html']['estimated_size'])) {
+                    $airsync_body->estimateddatasize = $message_body_data['html']['estimated_size'];
+                    $airsync_body->truncated = $message_body_data['html']['truncated'];
+                    $airsync_body->data = $message_body_data['html']['body']->stream;
+                    $eas_message->airsyncbasebody = $airsync_body;
+                }
                 $eas_message->airsyncbaseattachments = $imap_message->getAttachments($version);
             } elseif (isset($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN])) {
 
                 // Non MIME encoded plaintext
                 $this->_logger->info(sprintf(
                     '[%s] Sending PLAINTEXT Message.', $this->_procid));
-                $airsync_body->estimateddatasize = $message_body_data['plain']['size'];
-                $airsync_body->truncated = $message_body_data['plain']['truncated'];
-                $airsync_body->data = $message_body_data['plain']['body']->stream;
-                $airsync_body->type = Horde_ActiveSync::BODYPREF_TYPE_PLAIN;
-                $eas_message->airsyncbasebody = $airsync_body;
+                if (!empty($message_body_data['plain']['size'])) {
+                    $airsync_body->estimateddatasize = $message_body_data['plain']['size'];
+                    $airsync_body->truncated = $message_body_data['plain']['truncated'];
+                    $airsync_body->data = $message_body_data['plain']['body']->stream;
+                    $airsync_body->type = Horde_ActiveSync::BODYPREF_TYPE_PLAIN;
+                    $eas_message->airsyncbasebody = $airsync_body;
+                }
                 $eas_message->airsyncbaseattachments = $imap_message->getAttachments($version);
             }
         }
@@ -1110,7 +1100,7 @@ class Horde_ActiveSync_Imap_Adapter
         if ($version >= Horde_ActiveSync::VERSION_TWELVE) {
             $eas_message->contentclass = 'urn:content-classes:message';
             if ($mime_part = $imap_message->hasiCalendar()) {
-                $data = $mime_part->getContents();
+                $data = Horde_ActiveSync_Utils::ensureUtf8($mime_part->getContents(), $mime_part->getCharset());
                 $vCal = new Horde_Icalendar();
                 if ($vCal->parsevCalendar($data, 'VCALENDAR', $mime_part->getCharset())) {
                     try {
@@ -1362,6 +1352,9 @@ class Horde_ActiveSync_Imap_Adapter
     /**
      * Return the attendee participation status.
      *
+     * @param Horde_Icalendar $vCal  The vCalendar component.
+     * @param string $from           The From: address.
+     *
      * @param Horde_Icalendar
      * @throws Horde_ActiveSync_Exception
      */
@@ -1370,40 +1363,6 @@ class Horde_ActiveSync_Imap_Adapter
         foreach ($vCal->getComponents() as $key => $component) {
             switch ($component->getType()) {
             case 'vEvent':
-                try {
-                    $atname = $component->getAttribute('ATTENDEE');
-                } catch (Horde_Icalendar_Exception $e) {
-                    throw new Horde_ActiveSync_Exception($e);
-                }
-                // EAS only allows a single name per response.
-                if (is_array($atname)) {
-                    $atname = current($atname);
-                }
-
-                try {
-                    $version = $component->getAttribute('VERSION');
-                } catch (Horde_Icalendar_Exception $e) {
-                    throw new Horde_ActiveSync_Exception($e);
-                }
-                if ($version < 2) {
-                    $addr = new Horde_Mail_Rfc822_Address($atname);
-                    if (!$addr->isValid) {
-                        throw new Horde_ActiveSync_Exception('Invalid Email Address');
-                    }
-                    $attendee = Horde_String::lower($addr->bare_address);
-                } else {
-                    $attendee = str_replace('mailto:', '', Horde_String::lower($atname));
-                }
-
-                // Require the sender to be the same as the attendee.
-                $addr = new Horde_Mail_Rfc822_Address($from);
-                $from = Horde_String::lower($addr->bare_address);
-                if ($from != $attendee) {
-                    throw new Horde_ActiveSync_Exception(sprintf(
-                        'Unmatched sender and attendee: %s, %s',
-                        $from,
-                        $attendee));
-                }
                 try {
                     $atparams = $component->getAttribute('ATTENDEE', true);
                 } catch (Horde_Icalendar_Exception $e) {
@@ -1464,7 +1423,7 @@ class Horde_ActiveSync_Imap_Adapter
         }
         $ids = new Horde_Imap_Client_Ids($uids);
         try {
-            return $imap->fetch($mbox, $query, array('ids' => $ids));
+            return $imap->fetch($mbox, $query, array('ids' => $ids, 'exists' => true));
         } catch (Horde_Imap_Client_Exception $e) {
             $this->_logger->err(sprintf(
                 '[%s] Unable to fetch message: %s',
@@ -1493,82 +1452,6 @@ class Horde_ActiveSync_Imap_Adapter
         return false;
     }
 
-    /**
-     * Strip out non 7Bit characters from a text string.
-     *
-     * @param string $text  The string to strip.
-     *
-     * @return string|boolean  The stripped string, or false if failed.
-     */
-    protected function _stripNon7BitChars($text)
-    {
-        return preg_replace('/[^\x09\x0A\x0D\x20-\x7E]/', '', $text);
-    }
-
-    /**
-     * Ensure $data is converted to valid UTF-8 data. Works as follows:
-     * Converts to UTF-8, assuming data is in $from_charset encoding. If
-     * that produces invalid UTF-8, attempt to convert to most common mulitibyte
-     * encodings. If that *still* fails, strip out non 7-Bit characters...and
-     * force encoding to UTF-8 from $from_charset as a last resort.
-     *
-     * @param string $data          The string data to convert to UTF-8.
-     * @param string $from_charset  The character set to assume $data is encoded
-     *                              in.
-     *
-     * @return string  A valid UTF-8 encoded string.
-     */
-    protected function _validateUtf8($data, $from_charset)
-    {
-        $this->_logger->info(sprintf(
-            '[%s] Validating UTF-8 data coming from %s',
-            $this->_procid,
-            $from_charset));
-        $text = Horde_String::convertCharset($data, $from_charset, 'UTF-8');
-        if (!Horde_String::validUtf8($text)) {
-            $this->_logger->info(sprintf(
-                '[%s] Found invalid UTF-8 data, try different encodings.',
-                $this->_procid));
-            $test_charsets = array(
-                'windows-1252',
-                'UTF-8'
-            );
-            foreach ($test_charsets as $charset) {
-                if ($charset != $from_charset) {
-                    $text = Horde_String::convertCharset($data, $charset, 'UTF-8');
-                    if (Horde_String::validUtf8($text)) {
-                        $this->_logger->info(sprintf(
-                            '[%s] Found valid UTF-8 data when using %s',
-                            $this->_procid,
-                            $charset));
-                        return $text;
-                    }
-                }
-            }
-
-            // Invalid UTF-8 still found. Strip out non 7-bit characters, or if
-            // that fails, force a conersion to UTF-8 as a last resort. Need
-            // to break string into smaller chunks to avoid hitting
-            // https://bugs.php.net/bug.php?id=37793
-            $this->_logger->info(sprintf(
-                '[%s] Could not encode UTF-8 data. Removing non 7-bit characters.',
-                $this->_procid));
-            $chunk_size = 4000;
-            $text = '';
-            while ($data !== false && strlen($data)) {
-                $test = $this->_stripNon7BitChars(substr($data, 0, $chunk_size));
-                if ($test !== false) {
-                    $text .= $test;
-                } else {
-                    return Horde_String::convertCharset($data, $from_charset, 'UTF-8', true);
-                }
-                $data = substr($data, $chunk_size);
-            }
-        }
-
-        return $text;
-    }
-
     /**
      * Converts and validates the message body data structure.
      *
@@ -1586,14 +1469,14 @@ class Horde_ActiveSync_Imap_Adapter
         if (!empty($data['plain'])) {
             $stream = new Horde_Stream_Temp(array('max_memory' => 1048576));
             $filter_h = stream_filter_append($stream->stream, 'horde_eol', STREAM_FILTER_WRITE);
-            $stream->add($this->_validateUtf8($data['plain']['body'], $data['plain']['charset']), true);
+            $stream->add(Horde_ActiveSync_Utils::ensureUtf8($data['plain']['body'], $data['plain']['charset']), true);
             stream_filter_remove($filter_h);
             $data['plain']['body'] = $stream;
         }
         if (!empty($data['html'])) {
             $stream = new Horde_Stream_Temp(array('max_memory' => 1048576));
             $filter_h = stream_filter_append($stream->stream, 'horde_eol', STREAM_FILTER_WRITE);
-            $stream->add($this->_validateUtf8($data['html']['body'], $data['html']['charset']), true);
+            $stream->add(Horde_ActiveSync_Utils::ensureUtf8($data['html']['body'], $data['html']['charset']), true);
             stream_filter_remove($filter_h);
             $data['html']['body'] = $stream;
         }
