@@ -223,12 +223,12 @@ class Horde_ActiveSync_Imap_Message
         // Deduce which part(s) we need to request.
         $want_html_text = $version >= Horde_ActiveSync::VERSION_TWELVE &&
             (!empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_HTML]) ||
-            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_MIME]) ||
-            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_RTF]));
+            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_MIME]));
 
         $want_plain_text = $version == Horde_ActiveSync::VERSION_TWOFIVE ||
             empty($options['bodyprefs']) ||
             !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_PLAIN]) ||
+            !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_RTF]) ||
             !empty($options['bodyprefs'][Horde_ActiveSync::BODYPREF_TYPE_MIME]) ||
             ($want_html_text && empty($html_id));
 
@@ -458,7 +458,11 @@ class Horde_ActiveSync_Imap_Message
             $mime_part = $data;
         }
         $tnef_parser = Horde_Compress::factory('Tnef');
-        $tnef_data = $tnef_parser->decompress($mime_part->getContents());
+        try {
+            $tnef_data = $tnef_parser->decompress($mime_part->getContents());
+        } catch (Horde_Compress_Exception $e) {
+            return false;
+        }
         if (!count($tnef_data)) {
             return false;
         }
@@ -482,7 +486,7 @@ class Horde_ActiveSync_Imap_Message
     }
 
     /**
-     * Return an array of mime parts for each messages attachment.
+     * Return an array of mime parts for each message attachment.
      *
      * @return array An array of Horde_Mime_Part objects.
      */
@@ -493,6 +497,13 @@ class Horde_ActiveSync_Imap_Message
         foreach ($map as $id => $type) {
             if ($this->isAttachment($id, $type)) {
                 $mpart = $this->getMimePart($id);
+                // Work around PHP bug 65776
+                if ($mpart->getPrimaryType() == 'text') {
+                    $mpart->setContents(
+                        $mpart->replaceEOL(
+                            $mpart->getContents(array('stream' => true)), Horde_Mime_Part::RFC_EOL, true)
+                    );
+                }
                 if ($mpart->getType() == 'text/calendar') {
                     $mpart->setDisposition('inline');
                 }
@@ -805,6 +816,7 @@ class Horde_ActiveSync_Imap_Message
             }
             return false;
         case 'application/pkcs7-signature':
+        case 'application/x-pkcs7-signature':
             return false;
         }
 
@@ -877,7 +889,8 @@ class Horde_ActiveSync_Imap_Message
         if (empty($message)) {
             $message = $this->_message;
         }
-        if ($message->getType() == 'application/pkcs7-mime') {
+        if ($message->getType() == 'application/pkcs7-mime' ||
+            $message->getType() == 'application/x-pkcs7-mime') {
             return true;
         }
 
