@@ -164,7 +164,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
 
         try {
             $results = $this->_db->selectValues($sql,
-                array($this->_deviceInfo->id, $this->_devInfo->user, $uid));
+                array($this->_deviceInfo->id, $this->_deviceInfo->user, $uid));
         } catch (Horde_Db_Exception $e) {
             $this->_logger->err(sprintf(
                 '[%s] %s',
@@ -175,13 +175,14 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         }
         $update = 'UPDATE ' . $this->_syncStateTable . ' SET sync_data = ? WHERE '
             . 'sync_devid = ? AND sync_user = ? AND sync_folderid = ?';
+
         foreach ($results as $folder) {
             $folder = unserialize($folder);
             $folder->setServerId($serverid);
             $folder = serialize($folder);
             try {
                 $this->_db->update($update,
-                    array($folder, $this->_deviceInfo->id, $this->_devInfo->user, $uid));
+                    array($folder, $this->_deviceInfo->id, $this->_deviceInfo->user, $uid));
             } catch (Horde_Db_Exception $e) {
                 $this->_logger->err(sprintf(
                     '[%s] %s',
@@ -339,10 +340,10 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         } catch (Horde_Db_Exception $e) {
             // Might exist already if the last sync attempt failed.
             $this->_logger->notice(
-                sprintf('[%s] Error saving state for synckey %s: %s - removing previous sync state and trying again.',
+                sprintf('[%s] Previous request processing for synckey %s failed to be accepted by the client, removing previous state and trying again.',
                         $this->_procid,
-                        $this->_syncKey,
-                        $e->getMessage()));
+                        $this->_syncKey)
+                );
             $this->_db->delete('DELETE FROM ' . $this->_syncStateTable . ' WHERE sync_key = ?', array($this->_syncKey));
             $this->_db->insert($sql, $params);
         }
@@ -376,6 +377,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      * @param string $user      The current sync user, only needed if change
      *                          origin is CHANGE_ORIGIN_PIM
      * @param string $clientid  PIM clientid sent when adding a new message
+     *
+     * @todo This method needs some cleanup, abstraction.
      */
     public function updateState(
         $type, array $change, $origin = Horde_ActiveSync::CHANGE_ORIGIN_NA,
@@ -419,7 +422,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                     !empty($change['flags'])) {
                     $type = Horde_ActiveSync::CHANGE_TYPE_FLAGS;
                 }
-                if ($type == Horde_ActiveSync::CHANGE_TYPE_FLAGS) {
+                switch ($type) {
+                case Horde_ActiveSync::CHANGE_TYPE_FLAGS:
                     if (isset($change['flags']['read'])) {
                         // This is a mail sync changing only a read flag.
                         $sql = 'INSERT INTO ' . $this->_syncMailMapTable
@@ -434,19 +438,26 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
                             . ' VALUES (?, ?, ?, ?, ?, ?)';
                         $flag_value = !empty($change['flags']['flagged']);
                     }
-                } else {
+                    break;
+                case Horde_ActiveSync::CHANGE_TYPE_DELETE:
                     $sql = 'INSERT INTO ' . $this->_syncMailMapTable
                         . ' (message_uid, sync_key, sync_devid,'
                         . ' sync_folderid, sync_user, sync_deleted)'
                         . ' VALUES (?, ?, ?, ?, ?, ?)';
+                    $flag_value = true;
+                    break;
+                case Horde_ActiveSync::CHANGE_TYPE_CHANGE:
+                    // Used to remember "new" messages that are a result of
+                    // a MOVEITEMS request.
+                    $sql = 'INSERT INTO ' . $this->_syncMailMapTable
+                        . ' (message_uid, sync_key, sync_devid,'
+                        . ' sync_folderid, sync_user, sync_changed)'
+                        . ' VALUES (?, ?, ?, ?, ?, ?)';
+                    $flag_value = true;
+                    break;
                 }
-                $params = array(
-                    $change['id'],
-                    $syncKey,
-                    $this->_deviceInfo->id,
-                    $change['serverid'],
-                    $user,
-                    ($type == Horde_ActiveSync::CHANGE_TYPE_FLAGS) ? $flag_value : true
+                $params = array($change['id'], $syncKey, $this->_deviceInfo->id,
+                    $change['serverid'], $user, $flag_value
                 );
                 break;
 
@@ -533,6 +544,11 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             if (!$device = $this->_db->selectOne($query, array($devId))) {
                 throw new Horde_ActiveSync_Exception('Device not found.');
             }
+            $columns = $this->_db->columns($this->_syncDeviceTable);
+            $device['device_properties'] = $columns['device_properties']
+                ->binaryToString($device['device_properties']);
+            $device['device_supported'] = $columns['device_supported']
+                ->binaryToString($device['device_supported']);
         } catch (Horde_Db_Exception $e) {
             throw new Horde_ActiveSync_Exception($e);
         }
@@ -951,13 +967,19 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
             }
             if (!empty($device_query)) {
                 $this->_db->delete($device_query, $values);
-            } elseif (!empty($user_query) && empty($options['devId'])) {
-                // If there was a user_deletion, check if we should remove the
-                // device entry as well
-                $sql = 'SELECT COUNT(*) FROM ' . $this->_syncUsersTable . ' WHERE device_id = ?';
-                if (!$this->_db->selectValue($sql, array($options['devId']))) {
-                    $query = 'DELETE FROM ' . $this->_syncDeviceTable . ' WHERE device_id = ?';
-                    $this->_db->delete($query, array($options['devId']));
+            } elseif (!empty($user_query)) {
+                $sql = 'SELECT t1.device_id FROM horde_activesync_device t1 '
+                    . 'LEFT JOIN horde_activesync_device_users t2 '
+                    . 'ON t1.device_id = t2.device_id WHERE t2.device_id IS NULL';
+                try {
+                    $devids = $this->_db->selectValues($sql);
+                    foreach ($devids as $id) {
+                        $this->_db->delete(
+                            'DELETE FROM horde_activesync_device WHERE device_id = ?',
+                            array($id));
+                    }
+                } catch (Horde_Db_Exception $e) {
+                    throw new Horde_ActiveSync_Exception($e->getMessage());
                 }
             }
         } catch (Horde_Db_Exception $e) {
@@ -1004,8 +1026,9 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         $sql = 'SELECT cache_data FROM ' . $this->_syncCacheTable
             . ' WHERE cache_devid = ? AND cache_user = ?';
         try {
-            $data = $this->_db->selectValue(
-                $sql, array($devid, $user));
+            $data = $this->_db->selectValue($sql, array($devid, $user));
+            $columns = $this->_db->columns($this->_syncCacheTable);
+            $data = $columns['cache_data']->binaryToString($data);
         } catch (Horde_Db_Exception $e) {
             throw new Horde_ActiveSync_Exception($e);
         }
@@ -1136,10 +1159,11 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      * PIM-initiated change for the provided uid. Used to avoid mirroring back
      * changes to the PIM that it sent to the server.
      *
-     * @param array $changes  The changes array, containing 'id' and 'type'.
+     * @param array $changes  The changes array, each entry a hash containing
+     *                        'id' and 'type' keys.
      *
      * @return array  An array of UID -> timestamp of the last PIM-initiated
-     *                change for the specified uid, or null if none found.
+     *                change for the specified UIDs, or null if none found.
      */
     protected function _getPIMChangeTS(array $changes)
     {
@@ -1185,14 +1209,12 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      */
     protected function _havePIMChanges()
     {
-        $class = $this->_collection['class'];
-
-        $table = $class == Horde_ActiveSync::CLASS_EMAIL ?
-            $this->_syncMailMapTable :
-            $this->_syncMapTable;
-        $sql = 'SELECT COUNT(*) FROM ' . $table
+        // No benefit from making this extra query for email collections.
+        if ($this->_collection['class'] == Horde_ActiveSync::CLASS_EMAIL) {
+            return true;
+        }
+        $sql = 'SELECT COUNT(*) FROM ' . $this->_syncMapTable
             . ' WHERE sync_devid = ? AND sync_user = ? AND sync_folderid = ?';
-
         try {
             return (bool)$this->_db->selectValue(
                 $sql, array($this->_deviceInfo->id, $this->_deviceInfo->user, $this->_collection['serverid']));
@@ -1214,8 +1236,8 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
      */
     protected function _getMailMapChanges(array $changes)
     {
-        $sql = 'SELECT message_uid, sync_read, sync_flagged, sync_deleted FROM '
-            . $this->_syncMailMapTable
+        $sql = 'SELECT message_uid, sync_read, sync_flagged, sync_deleted,'
+            . 'sync_changed FROM ' . $this->_syncMailMapTable
             . ' WHERE sync_folderid = ? AND sync_devid = ?'
             . ' AND sync_user = ? AND message_uid IN '
             . '(' . implode(',', array_fill(0, count($changes), '?')) . ')';
@@ -1240,15 +1262,20 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         foreach ($rows as $row) {
             foreach ($changes as $change) {
                 if ($change['id'] == $row['message_uid']) {
-                    if ($change['type'] == Horde_ActiveSync::CHANGE_TYPE_FLAGS) {
+                    switch ($change['type']) {
+                    case Horde_ActiveSync::CHANGE_TYPE_FLAGS:
                         $results[$row['message_uid']][$change['type']] =
                             (!is_null($row['sync_read']) && $row['sync_read'] == $change['flags']['read']) ||
                             (!is_null($row['sync_flagged'] && $row['sync_flagged'] == $change['flags']['flagged']));
-                        continue 2;
-                    } elseif ($change['type'] == Horde_ActiveSync::CHANGE_TYPE_DELETE) {
+                        continue 3;
+                    case Horde_ActiveSync::CHANGE_TYPE_DELETE:
                         $results[$row['message_uid']][$change['type']] =
                             !is_null($row['sync_deleted']) && $row['sync_deleted'] == true;
-                        continue 2;
+                        continue 3;
+                    case Horde_ActiveSync::CHANGE_TYPE_CHANGE:
+                        $results[$row['message_uid']][$change['type']] =
+                            !is_null($row['sync_changed']) && $row['sync_changed'] == true;
+                        continue 3;
                     }
                 }
             }
@@ -1327,7 +1354,7 @@ class Horde_ActiveSync_State_Sql extends Horde_ActiveSync_State_Base
         // above.
         foreach (array($this->_syncMapTable, $this->_syncMailMapTable) as $table) {
             $remove = array();
-            $sql = 'SELECT sync_key FROM ' . $table
+            $sql = 'SELECT DISTINCT sync_key FROM ' . $table
                 . ' WHERE sync_devid = ? AND sync_user = ?';
 
             try {
