@@ -23,20 +23,21 @@
  * @copyright 2010-2014 Horde LLC (http://www.horde.org)
  * @author    Michael J Rubinsky <mrubinsk@horde.org>
  * @package   ActiveSync
+ * @internal  Not intended for use outside of the Horde_ActiveSync library.
  *
- * @property array folders              The folders cache.
- * @property integer hbinterval         The heartbeat interval (in seconds).
- * @property integer wait               The wait interval (in minutes).
- * @property integer pingheartbeat      The heartbeat used in PING requests.
- * @property string hierarchy           The hierarchy synckey.
- * @property array confirmed_synckeys   Array of synckeys being confirmed during
- *                                      a looping sync.
- * @property integer lastuntil          Timestamp representing the last planned
- *                                      looping sync end time.
- * @property integer lasthbsyncstarted  Timestamp of the start of the last
- *                                      looping sync.
- * @property integer lastsyncendnormal  Timestamp of the last looping sync that
- *                                      ended normally.
+ * @property array   $folders              The folders cache.
+ * @property integer   $hbinterval         The heartbeat interval (in seconds).
+ * @property integer   $wait               The wait interval (in minutes).
+ * @property integer   $pingheartbeat      The heartbeat used in PING requests.
+ * @property string   $hierarchy           The hierarchy synckey.
+ * @property array   $confirmed_synckeys   Array of synckeys being confirmed during
+ *                                       a looping sync.
+ * @property integer   $lastuntil          Timestamp representing the last planned
+ *                                       looping sync end time.
+ * @property integer   $lasthbsyncstarted  Timestamp of the start of the last
+ *                                       looping sync.
+ * @property integer   $lastsyncendnormal  Timestamp of the last looping sync that
+ *                                       ended normally.
  */
 class Horde_ActiveSync_SyncCache
 {
@@ -162,7 +163,7 @@ class Horde_ActiveSync_SyncCache
             array('lasthbsyncstarted', 'timestamp'));
 
         if ((!$hb_only && $cache['timestamp'] > $this->_data['timestamp']) ||
-            ($cache['lasthbsyncstarted'] > $this->_data['lasthbsyncstarted'])) {
+            (!empty($cache['lasthbsyncstarted']) && $cache['lasthbsyncstarted'] > $this->_data['lasthbsyncstarted'])) {
             return false;
         }
 
@@ -366,7 +367,7 @@ class Horde_ActiveSync_SyncCache
             $this->_user,
             array('collections')
         );
-        $cache_collections = $syncCache['collections'];
+        $cache_collections = !is_array($syncCache['collections']) ? array() : $syncCache['collections'];
         foreach ($cache_collections as $id => $cache_collection) {
             if (!isset($cache_collection['lastsynckey'])) {
                 continue;
