@@ -161,6 +161,22 @@ class Horde_ActiveSync_StateTest_Sql_Base extends Horde_ActiveSync_StateTest_Bas
         $this->_testHierarchy();
     }
 
+    /**
+     * @depends testGetDeviceInfo
+     */
+    public function testListDevices()
+    {
+        $this->_testListDevices();
+    }
+
+    /**
+     * @depends testListDevices
+     */
+    public function testPolicyKeys()
+    {
+        $this->_testPolicyKeys();
+    }
+
     /**
      * @depends testCollectionHandler
      */
@@ -169,7 +185,6 @@ class Horde_ActiveSync_StateTest_Sql_Base extends Horde_ActiveSync_StateTest_Bas
         $this->_testPartialSyncWithOnlyChangedHbInterval();
     }
 
-
     public static function setUpBeforeClass()
     {
         $dir = dirname(__FILE__) . '/../../../../../migration/Horde/ActiveSync';
@@ -181,20 +196,22 @@ class Horde_ActiveSync_StateTest_Sql_Base extends Horde_ActiveSync_StateTest_Bas
             error_reporting(E_ALL | E_STRICT);
         }
         self::$logger = new Horde_Test_Log();
-        self::$migrator = new Horde_Db_Migration_Migrator(
-            self::$db,
-            self::$logger->getLogger(),
-            array('migrationsPath' => $dir,
-                  'schemaTableName' => 'horde_activesync_test_schema'));
-        self::$migrator->up();
+        if (self::$db) {
+            self::$migrator = new Horde_Db_Migration_Migrator(
+                self::$db,
+                self::$logger->getLogger(),
+                array('migrationsPath' => $dir,
+                      'schemaTableName' => 'horde_activesync_schema'));
+            self::$migrator->up();
+        }
     }
 
     public static function tearDownAfterClass()
     {
-        if (self::$migrator) {
-            self::$migrator->down();
-        }
         if (self::$db) {
+            if (self::$migrator) {
+                self::$migrator->down();
+            }
             self::$db->disconnect();
             self::$db = null;
         }
@@ -205,7 +222,11 @@ class Horde_ActiveSync_StateTest_Sql_Base extends Horde_ActiveSync_StateTest_Bas
     {
         if (!self::$db) {
             $this->markTestSkipped(self::$reason);
+            return;
         }
         self::$state = new Horde_ActiveSync_State_Sql(array('db' => self::$db));
+        $backend = $this->getMockSkipConstructor('Horde_ActiveSync_Driver_Base');
+        $backend->expects($this->any())->method('getUser')->will($this->returnValue('mike'));
+        self::$state->setBackend($backend);
     }
 }
\ No newline at end of file
